# Copyright 2013 David Reichard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###############################################################################

import random

import pygame

from gamelib.director import director
from gamelib.scene import Scene
from gamelib.scene_object import SceneObject
from game.player import Player, PlayerSprite
from gamelib.timer import Timer
import gamelib.data
import game.entities
from game.entities import Shark, Sponge, Jellyfish, Net

LEVEL_MUSIC = [
    'amb_1',
    'amb_2',
    'amb_3',
    'final'
]

LEVEL_SCORE_LIMITS = [
    2000,
    4000,
    8000,
    16000,
    32000,
    64000,
    128000,
    256000,
    512000,
    1240000
]

DISPLAY_SURFACE = pygame.display.get_surface()
DISPLAY_WIDTH = DISPLAY_SURFACE.get_width()
DISPLAY_HEIGHT = DISPLAY_SURFACE.get_height()

SCORE_FONT = gamelib.data.load_font("PARPG", 18)
SCOREBAR_COLOR = (255, 255, 255)

HEART_FULL = gamelib.data.load_image("heart_full.png", None, True)
HEART_EMPTY = gamelib.data.load_image("heart_empty.png", None, True)
SKY_ISLANDS = gamelib.data.load_image("sky_islands.png", None, True)
SEA = gamelib.data.load_image("sea.png", None, True)
GROUND = gamelib.data.load_image("ground.png", None, True)

# Parallax Scrolling Speeds
BACK_LAYER_SPEED = 25
MIDDLE_LAYER_SPEED = 55
FRONT_LAYER_SPEED = 90

LEVEL_TIME = 120 # 2 minutes per level

breath_sprites = {0: gamelib.data.load_image("breath_meter_0.png", None, True),
                  5: gamelib.data.load_image("breath_meter_5.png", None, True),
                  10: gamelib.data.load_image("breath_meter_10.png", None, True),
                  15: gamelib.data.load_image("breath_meter_15.png", None, True),
                  20: gamelib.data.load_image("breath_meter_20.png", None, True),
                  25: gamelib.data.load_image("breath_meter_25.png", None, True),
                  30: gamelib.data.load_image("breath_meter_30.png", None, True),
                  35: gamelib.data.load_image("breath_meter_35.png", None, True),
                  40: gamelib.data.load_image("breath_meter_40.png", None, True),
                  45: gamelib.data.load_image("breath_meter_45.png", None, True),
                  50: gamelib.data.load_image("breath_meter_50.png", None, True),
                  55: gamelib.data.load_image("breath_meter_55.png", None, True),
                  60: gamelib.data.load_image("breath_meter_60.png", None, True),
                  65: gamelib.data.load_image("breath_meter_65.png", None, True),
                  70: gamelib.data.load_image("breath_meter_70.png", None, True),
                  75: gamelib.data.load_image("breath_meter_75.png", None, True),
                  80: gamelib.data.load_image("breath_meter_80.png", None, True),
                  85: gamelib.data.load_image("breath_meter_85.png", None, True),
                  90: gamelib.data.load_image("breath_meter_90.png", None, True),
                  95: gamelib.data.load_image("breath_meter_95.png", None, True),
                  100: gamelib.data.load_image("breath_meter_100.png", None, True)}

breath_range = (0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85, 90, 95, 100)


class ScoreBar(SceneObject):
    def __init__(self, surface, pos):
        SceneObject.__init__(self, surface, None, pos)

    def on_add(self, parent):
        self.internal_score = 0
        parent.parent.listen("update", self.on_update)

    def on_update(self, dt):
        if Player.score != self.internal_score:
            self.image = SCORE_FONT.render("Level: {0}  Score: {1}".format(self.parent.parent.difficulty, Player.score),
                                           True, (SCOREBAR_COLOR))
            self.rect.left = 0
            self.internal_score = Player.score


class BreathBar(SceneObject):
    def update(self, dt):
        for num in breath_range:
            if Player.breath <= num:
                self.image = breath_sprites[num]
                return


class LifeHeart(SceneObject):
    def on_add(self, parent):
        self.full = True

    def update(self, dt):
        if self.full and self.image != HEART_FULL:
            self.image = HEART_FULL
        elif not self.full and self.image != HEART_EMPTY:
            self.image = HEART_EMPTY


class Level(Scene):
    def on_create(self, *args, **kwargs):
        self.difficulty = kwargs["difficulty"]
        self.time = LEVEL_TIME
        self.paused = False

        # Set up background layers
        self.add("back_layer", [SceneObject(SKY_ISLANDS), SceneObject(SKY_ISLANDS, None, (SKY_ISLANDS.get_width(), 0))])
        self.add("middle_layer", [SceneObject(SEA), SceneObject(SEA, None, (SEA.get_width(), 0))])
        self.add("front_layer", [SceneObject(GROUND), SceneObject(GROUND, None, (GROUND.get_width(), 0))])

        # Set up breath bar and health hearts
        # Breath Bar
        start_pos = pygame.display.get_surface().get_rect().topright
        self.add("breath", BreathBar(gamelib.data.load_image("breath_meter_0.png", None, True), None,
                                     (start_pos[0]-1, start_pos[1]+1), "topright"))

        # Player sprite
        self.player_sprite = PlayerSprite(None, gamelib.data.load_animation_set("turtle"),
                                          (0, DISPLAY_HEIGHT / 2 - 20), "center")
        self.add("player", self.player_sprite)

        # Create entity groups
        self.add("nets")
        self.add("sponges")
        self.add("jellies")
        self.add("enemies")

        # Health Hearts
        self.life_hearts = []
        start_pos = (start_pos[0]-5, start_pos[1]+20)
        for point in range(Player.max_health):
            self.life_hearts.insert(0, LifeHeart(HEART_FULL, None, start_pos, "topright"))
            start_pos = (start_pos[0]-40, start_pos[1])
        self.add("life", self.life_hearts)

        # Score Bar
        surface = SCORE_FONT.render("Level: {0}  Score: {1}".format(self.difficulty, Player.score),
                                    True, SCOREBAR_COLOR)
        self.add("score", ScoreBar(surface, (0, 0)))

        self.listen("update", Player.update)
        self.listen("update", self.on_update)
        self.listen("keydown", self.on_key_press)
        gamelib.data.load_music(LEVEL_MUSIC[random.randint(0, 2)])
        pygame.mixer.music.play(-1)

    # Entity spawn timers
    @Timer(5)
    def spawn_net(self):
        if random.randint(0, 100) < 60:
            self.add("nets", Net((DISPLAY_WIDTH, 40)))

        if self.difficulty > 3:
            if random.randint(0, 100) < 40:
                self.add("nets", Net((DISPLAY_WIDTH + 30, 40)))

        if self.difficulty > 6:
            if random.randint(0, 100) < 30:
                self.add("nets", Net((DISPLAY_WIDTH + 75, 40)))

        if self.difficulty > 9:
            if random.randint(0, 100) < 25:
                self.add("nets", Net((DISPLAY_WIDTH + 115, 40)))

    @Timer(3)
    def spawn_shark(self):
        if random.randint(0, 100) < 75:
            self.add("enemies", Shark((DISPLAY_WIDTH + game.entities.SHARK_IMAGE.get_width()/2, random.randint(70, 450))))
        if self.difficulty > 3:
            if random.randint(0, 100) < 65:
                self.add("enemies", Shark((DISPLAY_WIDTH + game.entities.SHARK_IMAGE.get_width()/2,
                         random.randint(70, 450))))
        if self.difficulty > 6:
            if random.randint(0, 100) < 55:
                self.add("enemies", Shark((DISPLAY_WIDTH + game.entities.SHARK_IMAGE.get_width()/2,
                                           random.randint(70, 450))))
        if self.difficulty > 9:
            if random.randint(0, 100) < 45:
                self.add("enemies", Shark((DISPLAY_WIDTH + game.entities.SHARK_IMAGE.get_width()/2,
                                           random.randint(70, 450))))
    @Timer(2)
    def spawn_jelly(self):
        if random.randint(0, 100) < 75:
            self.add("jellies", Jellyfish((DISPLAY_WIDTH, random.randint(70, 400))))

    @Timer(2.5)
    def spawn_sponge(self):
        if random.randint(0, 100) < 65:
            self.add("sponges", Sponge((DISPLAY_WIDTH, random.randint(400, 430))))

    def on_update(self, dt):
        # Have we reached our level limit? Then move on to the next one!
        if Player.score >= LEVEL_SCORE_LIMITS[self.difficulty - 1]:
            pygame.mixer.music.fadeout(500)
            director.replace(Win(self.difficulty + 1))

        # Did the player die? That is unfortunate...
        if Player.health <= 0:
            pygame.mixer.music.fadeout(500)
            Player.health = Player.max_health
            Player.breath = 100
            Player.score = 0
            director.replace(Lose())

        # update life hearts
        for heart in self.life_hearts[:Player.health]:
            if not heart.full:
                heart.full = True
        health_difference = Player.max_health - Player.health
        if health_difference > 0:
            for heart in self.life_hearts[-health_difference:]:
                if heart.full:
                    heart.full = False

        # parallax scrolling background
        for sprite in self.groups["back_layer"].sprites():
            sprite.translate(-BACK_LAYER_SPEED * dt, 0)
            if sprite.rect.right <= 0:
                sprite.rect.left = sprite.rect.width
        for sprite in self.groups["middle_layer"].sprites():
            sprite.translate(-MIDDLE_LAYER_SPEED * dt, 0)
            if sprite.rect.right <= 0:
                sprite.rect.left = sprite.rect.width
        for sprite in self.groups["front_layer"].sprites():
            sprite.translate(-FRONT_LAYER_SPEED * dt, 0)
            if sprite.rect.right <= 0:
                sprite.rect.left = sprite.rect.width

        for s in self.groups["nets"].sprites():
            s.translate(-MIDDLE_LAYER_SPEED * dt, 0)

        for s in self.groups["sponges"].sprites():
            s.translate(-FRONT_LAYER_SPEED * dt, 0)

        # spawn entities
        self.spawn_net(dt)
        self.spawn_shark(dt)
        self.spawn_sponge(dt)
        self.spawn_jelly(dt)

        # get rid of non-visible entities
        for s in self.groups["enemies"].sprites():
            if s.rect.right <= 0:
                s.kill()
        for s in self.groups["sponges"].sprites():
            if s.rect.right <= 0:
                s.kill()
        for s in self.groups["jellies"].sprites():
            if s.rect.right <= 0:
                s.kill()

        # collision handling
        for s in pygame.sprite.spritecollide(self.player_sprite, self.groups["enemies"], False,
                                             pygame.sprite.collide_rect_ratio(0.6)):
            s.on_collide()

        for s in pygame.sprite.spritecollide(self.player_sprite, self.groups["sponges"], False,
                                             pygame.sprite.collide_rect_ratio(0.6)):
            s.on_collide()

        for s in pygame.sprite.spritecollide(self.player_sprite, self.groups["jellies"], False,
                                             pygame.sprite.collide_rect_ratio(0.6)):
            s.on_collide()

        for s in pygame.sprite.spritecollide(self.player_sprite, self.groups["nets"], False,
                                             pygame.sprite.collide_rect_ratio(0.7)):
            s.on_collide()

    def on_key_press(self, unicode, key, mod):
        if key == pygame.K_SPACE:
            director.push(Paused())


class Paused(Scene):
    def on_create(self):
        self.listen("keydown", self.on_key_press)
        font = gamelib.data.load_font("PARPG", 60)
        self.add("text", SceneObject(font.render("PAUSED", False, (255, 255, 255)), None,
                                     (DISPLAY_WIDTH/2, DISPLAY_HEIGHT/2), 'center'))

    def on_key_press(self, unicode, key, mod):
        if key == pygame.K_SPACE:
            director.pop()


class Win(Scene):
    def on_create(self, difficulty):
        Player.health = Player.max_health
        Player.breath = 100
        self.next_difficulty = difficulty
        gamelib.data.load_music("win")
        pygame.mixer.music.play()

        self.listen("keydown", self.on_key_press)
        font = gamelib.data.load_font("PARPG", 60)
        font2 = gamelib.data.load_font("PARPG", 32)
        self.add("text", SceneObject(font.render("Success!", False, (255, 255, 255)), None,
                                     (DISPLAY_WIDTH/2, DISPLAY_HEIGHT/2-60), 'center'))
        self.add("text", SceneObject(font2.render("Press space to continue...", False, (255, 255, 255)), None,
                                     (DISPLAY_WIDTH/2, DISPLAY_HEIGHT/2), 'center'))

    def on_key_press(self, unicode, key, mod):
        if key == pygame.K_SPACE:
            pygame.mixer.music.fadeout(500)
            director.replace(Level(difficulty=self.next_difficulty))


class Lose(Scene):
    def on_create(self):
        gamelib.data.load_music("lost")
        pygame.mixer.music.play()
        self.listen("keydown", self.on_key_press)
        font = gamelib.data.load_font("PARPG", 60)
        font2 = gamelib.data.load_font("PARPG", 32)
        self.add("text", SceneObject(font.render("Game Over :(", False, (255, 255, 255)), None,
                                     (DISPLAY_WIDTH/2, DISPLAY_HEIGHT/2-60), 'center'))
        self.add("text", SceneObject(font2.render("Press space to continue...", False, (255, 255, 255)), None,
                                     (DISPLAY_WIDTH/2, DISPLAY_HEIGHT/2), 'center'))

    def on_key_press(self, unicode, key, mod):
        if key == pygame.K_SPACE:
            pygame.mixer.music.fadeout(500)
            director.pop()