# Copyright 2013 David Reichard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###############################################################################

from gamelib.scene import Scene
from gamelib.scene_object import SceneObject
from gamelib.director import director
from game.level import Level
import gamelib.data
import pygame
import sys

inactive_font = gamelib.data.load_font('PARPG', 32)
active_font = gamelib.data.load_font('PARPG', 39)
select_sound = gamelib.data.load_sound('select', 'ogg')
confirm_sound = gamelib.data.load_sound('confirm')

font_color = (150, 150, 0)
a_font_color = (200, 200, 0)


def quit_game():
    pygame.quit()
    sys.exit()


def howto_play():
    director.push(HowToPlay())


def new_game():
    director.push(Level(difficulty=1))


class HowToPlay(Scene):
    def on_create(self):
        self.add("image", SceneObject(gamelib.data.load_image('howtoplay.png', None, True)))
        self.listen("keydown", self.on_key_press)

    def on_key_press(self, unicode, key, mod):
        director.pop()


class MenuItem(SceneObject):
    def __init__(self, name, callback, pos, manipulation='topleft'):
        self.name = name
        self.callback = callback
        SceneObject.__init__(self, inactive_font.render(name, True, font_color), pos=pos, manipulation=manipulation)

    def make_inactive(self):
        self.image = inactive_font.render(self.name, True, font_color)

    def make_active(self):
        self.image = active_font.render(self.name, True, a_font_color)


class Menu(Scene):
    def on_create(self):
        self.add('background', SceneObject(gamelib.data.load_image('menu.jpg')))
        center = pygame.display.get_surface().get_rect().center

        self.new_item = MenuItem('New Game', new_game, (center[0], center[1]-25), 'center')
        self.howto_play = MenuItem('How to Play', howto_play, (center[0], center[1]+25), 'center')
        self.quit_item = MenuItem('Quit', quit_game, (center[0], center[1]+75), 'center')
        self.add('menu_items', (self.new_item, self.howto_play, self.quit_item))

        self.new_item.make_active()
        self.active_item_index = 0

        current = self.groups['menu_items'].get_sprite(self.active_item_index)
        self.turtle = SceneObject(None, gamelib.data.load_animation_set('turtle'),
                                  (current.rect.left - 20, current.rect.top + 10), 'topright')
        self.turtle.animation.set('swim')
        self.turtle.animation.start(speed=0.1)
        self.add('test', self.turtle)

        self.listen('scene_enter', self.on_enter)
        self.listen('keydown', self.on_key_press)

    def on_enter(self):
        gamelib.data.load_music('menu')
        pygame.mixer.music.play(-1)

    def on_key_press(self, unicode, key, mod):
        last_item = self.groups['menu_items'].get_sprite(self.active_item_index)
        if key == pygame.K_DOWN:
            self.active_item_index += 1
            if self.active_item_index >= len(self.groups['menu_items']):
                self.active_item_index = 0
        elif key == pygame.K_UP:
            self.active_item_index -= 1
            if self.active_item_index < 0:
                self.active_item_index = len(self.groups['menu_items']) - 1

        if key == pygame.K_DOWN or key == pygame.K_UP:
            select_sound.play()
            last_item.make_inactive()
            current = self.groups['menu_items'].get_sprite(self.active_item_index)
            current.make_active()
            self.turtle.move(current.rect.left - 20, current.rect.top + 10)

        if key == pygame.K_RETURN or key == pygame.K_SPACE:
            confirm_sound.play()
            pygame.mixer.music.fadeout(500)
            self.groups['menu_items'].get_sprite(self.active_item_index).callback()