# Copyright 2013 David Reichard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###############################################################################


import pygame
from gamelib.scene_object import SceneObject
import gamelib.data
from gamelib.timer import Timer
from gamelib.vector import Vec2d

SWIM_ANIMATION_BASE_SPEED = 0.1
SWIM_ANIMATION_INCREASED_SPEED = 0.05
PLAYER_MAX_SPEED = 200
PLAYER_ACCELERATION_SPEED = 600
PLAYER_FRICTION = 175
PLAYER_BOUNDS = pygame.Rect(-5, 25, 600, 440)

BREATH_TIMER = 12  # seconds before out of breath
BREATH_INTERVAL = BREATH_TIMER / 20.0
BREATH_REFRESH_INTERVAL = 50  # breath points per second
BREATH_DAMAGE_TIMER = 2  # seconds per heart decrease
SURFACE_SOUND = gamelib.data.load_sound('splash1')
DIVE_SOUND = gamelib.data.load_sound('splash2')
BREATH_HIT = gamelib.data.load_sound('breath_hit')

class Player(object):
    health = 5
    max_health = 5
    breath = 100
    breath_timer = 0.0
    breath_damage_timer = 0.0
    score = 0
    surfaced = False

    @classmethod
    def heal(cls, amount):
        cls.health = min(cls.health + amount, cls.max_health)

    @classmethod
    def damage(cls, amount):
        cls.health = max(cls.health - amount, 0)

    @classmethod
    def increase_score(cls, amount):
        cls.score += amount

    @classmethod
    @Timer(BREATH_INTERVAL)
    def decrease_breath(cls):
        cls.breath = max(cls.breath-5, 0)

    @classmethod
    def update(cls, dt):
        if cls.surfaced and not cls.breath == 100:
            cls.breath = min(cls.breath + (BREATH_REFRESH_INTERVAL * dt), 100)
        elif not cls.surfaced:
            cls.decrease_breath(dt)

        if cls.breath <= 0:
            cls.breath_damage_timer += dt
            if cls.breath_damage_timer >= BREATH_DAMAGE_TIMER:
                cls.breath_damage_timer -= BREATH_DAMAGE_TIMER
                cls.damage(1)
                BREATH_HIT.play()
        else:
            if cls.breath_damage_timer > 0:
                cls.breath_damage_timer = 0


class PlayerSprite(SceneObject):
    def on_add(self, parent):
        parent.parent.listen("keydown", self.on_key_press)
        parent.parent.listen("keyup", self.on_key_release)
        parent.parent.listen("update", self.on_update)
        self.velocity = Vec2d(0, 0)
        self.animation.set("swim")
        self.animation.start(SWIM_ANIMATION_BASE_SPEED)

    def on_key_press(self, unicode, key, mod):
        pass

    def on_key_release(self, key, mod):
        pass

    def on_update(self, dt):
        self.translate(*(self.velocity * dt))
        #player control
        keys = pygame.key.get_pressed()
        if keys[pygame.K_DOWN]:
            self.velocity.y += PLAYER_ACCELERATION_SPEED * dt
        if keys[pygame.K_UP]:
            self.velocity.y -= PLAYER_ACCELERATION_SPEED * dt
        if keys[pygame.K_LEFT]:
            self.velocity.x -= PLAYER_ACCELERATION_SPEED * dt
        if keys[pygame.K_RIGHT]:
            self.velocity.x += PLAYER_ACCELERATION_SPEED * dt

        # Increase animation speed when actively swimming.
        if keys[pygame.K_DOWN] or keys[pygame.K_UP] or keys[pygame.K_LEFT] or keys[pygame.K_RIGHT]:
            if self.animation.speed != SWIM_ANIMATION_INCREASED_SPEED:
                self.animation.speed = SWIM_ANIMATION_INCREASED_SPEED
        else:
            if self.animation.speed != SWIM_ANIMATION_BASE_SPEED:
                self.animation.speed = SWIM_ANIMATION_BASE_SPEED

        # Don't want to go too fast.
        if self.velocity.x < 0:
            x_mod = -1
        else:
            x_mod = 1
        if self.velocity.y < 0:
            y_mod = -1
        else:
            y_mod = 1
        self.velocity.x = min(abs(self.velocity.x), PLAYER_MAX_SPEED) * x_mod
        self.velocity.y = min(abs(self.velocity.y), PLAYER_MAX_SPEED) * y_mod

        # Add some friction against Penelope
        if self.velocity.x > 0:
            self.velocity.x -= PLAYER_FRICTION * dt
        if self.velocity.y > 0:
            self.velocity.y -= PLAYER_FRICTION * dt
        if self.velocity.x < 0:
            self.velocity.x += PLAYER_FRICTION * dt
        if self.velocity.y < 0:
            self.velocity.y += PLAYER_FRICTION * dt

        # Keep Penelope in bounds.
        new_rect = self.rect.clamp(PLAYER_BOUNDS)
        if new_rect.x != self.rect.x:
            self.velocity.x = 0
        if new_rect.y != self.rect.y:
            self.velocity.y = 0
        self.rect = new_rect

        # Let Penelope surface for air.
        if self.rect.top <= PLAYER_BOUNDS.top + 1 and not Player.surfaced:
            Player.surfaced = True
            SURFACE_SOUND.play()

        elif self.rect.top > PLAYER_BOUNDS.top + 1 and Player.surfaced:
            Player.surfaced = False
            DIVE_SOUND.play()